@@ -33,20 +33,28 @@ from sklearn.metrics import f1_score, classification_report, confusion_matrix
 
 from . import counter
 from . import nans
+from . import encoder
+from . import splitter
+
 
 
 class Snaplib__222:
     '''
     data preprocessing library
     '''
-    def __init__(self, start=0):
-        self.counter_cross_val = counter.Counter(start)
+
+    ENCODER = encoder.Encoder()
+
+
+
+
+
 
 
 
 
     @staticmethod
-    def nan_info(df:pd.DataFrame) -> pd.DataFrame:
+    def nan_info(df : pd.DataFrame) -> pd.DataFrame:
         '''
         Get pandas.DataFrame with information about missing data in columns
         Use case:
@@ -55,13 +63,13 @@ class Snaplib__222:
 
         if not isinstance(df, pd.core.frame.DataFrame):
             raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
-        return nans.Nans().nan_info(df)
+        return nans.nan_info(df)
 
 
 
 
     @staticmethod
-    def nan_plot(df:pd.DataFrame) -> plt.figure:
+    def nan_plot(df : pd.DataFrame) -> plt.figure:
         '''
         Visualise missing data in pandas.DataFrame
         Use case:
@@ -70,15 +78,16 @@ class Snaplib__222:
 
         if not isinstance(df, pd.core.frame.DataFrame):
             raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
-        return nans.Nans().nan_plot(df)
+        return nans.nan_plot(df)
+
 
 
 
-    
     @staticmethod
-    def cleane(df:pd.DataFrame, 
-                target:str = None, 
-                verbose:bool = True) -> pd.DataFrame:
+    def cleane( df : pd.DataFrame, 
+                target : str = None, 
+                verbose : bool = True
+                ) -> pd.DataFrame: 
         '''
         drop_duplicates
         drop rows with nan in target
@@ -94,7 +103,182 @@ class Snaplib__222:
                 raise TypeError('target must be of str type.')
         if type(verbose) != int and type(verbose) != bool:
             raise TypeError('verbose must be of int type or bool.')
-        return nans.Nans().cleane(df, target, verbose)
+        return nans.cleane(df, target, verbose)
+
+
+
+
+    # @staticmethod
+    def encode_dataframe(self, 
+                        df : pd.DataFrame, 
+                        inplace : bool = False, 
+                        ) -> pd.DataFrame: 
+        '''
+        encode a dataframe
+        Use case:
+        df = Snaplib().encode_dataframe(df)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if not isinstance(inplace, bool):
+            raise TypeError('The df must be a bool type instance.')
+
+
+        if inplace is False:
+            df_copied = df.copy()
+
+            return self.ENCODER.encode_dataframe(df_copied)
+        elif inplace is True:
+            return self.ENCODER.encode_dataframe(df)
+
+
+        
+
+    # @staticmethod
+    def decode_dataframe(self, 
+                        df : pd.DataFrame, 
+                        inplace : bool = False, 
+                        ) -> pd.DataFrame: 
+        '''
+        encode a dataframe
+        Use case:
+        df = Snaplib().decode_dataframe(df)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if not isinstance(inplace, bool):
+            raise TypeError('The df must be a bool type instance.')
+
+        if inplace is False:
+            df_copied = df.copy()
+            return self.ENCODER.decode_dataframe(df_copied)
+        elif inplace is True:
+            return self.ENCODER.decode_dataframe(df)
+
+
+
+
+    @staticmethod
+    def k_folds_split(df : pd.DataFrame, 
+                      target_name : str, 
+                      k : int, 
+                      ) -> dict: 
+
+        '''
+        Return a dictionary list of DataFrames and Series for target
+        with next structure:
+
+        k_fold_dict = { 
+                        'train_X' : [train_df_0, train_df_1, ... , train_df_k],
+                        'test_X'  : [etc.], 
+                        'train_y' : [series_0, series_1, ... , series_k],
+                        'test_y'  : [etc.],
+                      }
+
+        Use case:
+        k_fold_dict_data = Snaplib().k_folds_split( df, target_name, k)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if type(target_name) != str:
+            raise TypeError('target_name must be of str type.')
+        if type(k) != int:
+            raise TypeError('k must be of int type.')
+
+        return splitter.k_folds_split(df, target_name, k)
+
+
+
+
+
+    @staticmethod
+    def train_test_split_balanced(  df : pd.DataFrame, 
+                                    target_feature : str, 
+                                    test_size : float = 0.2, 
+                                    random_state: int = 0, 
+                                    research : bool = False, 
+                                    ) -> tuple:
+        ''' 
+        Split the data with the distribution as close as possible 
+        to the same in both the train set and the test set, and not only for the target column, 
+        but also for all other columns.
+
+        Use case:
+        train_X, test_X, train_y, test_y = Snaplib().train_test_split_balanced(df, target_name, test_size=0.2, random_state=0, research=True)
+
+        1) The input should be a whole DataFrame. It's the first positional argument.
+        2) And the second positional argument should be the name of the target feature as a string.
+        3) This method has internal testing.
+        In this way you can testing the usefulness of the custom split.
+        The first test performing with a random_state values from 0 to 1/test_size by sklearn.model_selection.train_test_split.
+        Before split performing a final testing with Snaplib().train_test_split_balanced.
+
+        You can perform testing by specifying the value of the research argument = True.
+
+        4) If you are convinced that the method is useful. You can silence the method.
+        Set the research argument to False.
+
+        5) The number of possible random_state is an equivalent to 1/test_size.
+
+        TESTS on https://www.kaggle.com/artyomkolas/train-test-split-balanced-custom-in-snaplib/notebook
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if type(target_feature) != str:
+            raise TypeError('target_feature must be of str type.')
+        if type(test_size) != float:
+            raise TypeError('test_size must be of float type in [0.0, 1.0].')
+        if type(random_state) != int:
+            raise TypeError('random_state must be of int type.')
+        if type(research) != bool:
+            raise TypeError('research must be of bool type.')
+
+        return  splitter.train_test_split_balanced( df, 
+                                                    target_feature, 
+                                                    test_size, 
+                                                    random_state, 
+                                                    research)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -163,24 +347,24 @@ class Snaplib:
 
 
 
-    # def nan_info(self, df):
-    #     '''
-    #     Get pandas.DataFrame with information about missing data in columns
-    #     Use case:
-    #     missing_info_df = Snaplib().nan_info(df)
-    #     '''
-    #     if not isinstance(df, pd.core.frame.DataFrame):
-    #         raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+    def nan_info(self, df):
+        '''
+        Get pandas.DataFrame with information about missing data in columns
+        Use case:
+        missing_info_df = Snaplib().nan_info(df)
+        '''
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
 
-    #     data_info = pd.DataFrame(index=df.columns)
-    #     try:
-    #         data_info['NaN_counts'] = df[[col for col in df.columns if df[col].isna().sum() > 0]].isna().sum().sort_values(ascending = True)
-    #         data_info['NaN_percent'] = data_info['NaN_counts'].apply(lambda x: round((x/len(df))*100, 2))
-    #         data_info['col_type'] = df.dtypes
-    #         data_info = data_info.sort_values(by=['NaN_counts'], ascending=True)
-    #     except:
-    #         return data_info
-    #     return data_info
+        data_info = pd.DataFrame(index=df.columns)
+        try:
+            data_info['NaN_counts'] = df[[col for col in df.columns if df[col].isna().sum() > 0]].isna().sum().sort_values(ascending = True)
+            data_info['NaN_percent'] = data_info['NaN_counts'].apply(lambda x: round((x/len(df))*100, 2))
+            data_info['col_type'] = df.dtypes
+            data_info = data_info.sort_values(by=['NaN_counts'], ascending=True)
+        except:
+            return data_info
+        return data_info
 
 
 
@@ -191,22 +375,22 @@ class Snaplib:
 
 
 
-    # def nan_plot(self, df):
-    #     '''
-    #     Visualise missing data in pandas.DataFrame
-    #     Use case:
-    #     Snaplib().nan_plot(df)
-    #     '''
+    def nan_plot(self, df):
+        '''
+        Visualise missing data in pandas.DataFrame
+        Use case:
+        Snaplib().nan_plot(df)
+        '''
 
-    #     if not isinstance(df, pd.core.frame.DataFrame):
-    #         raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
 
-    #     plt.figure(figsize=(int(len(df.columns)/4) if len(df.columns)>30 else 10, 10))
-    #     plt.pcolor(df.isnull(), cmap='Blues_r')
-    #     plt.yticks([int(el*(len(df)/10)) for el in range(0, 10)])
-    #     plt.xticks(np.arange(0.5, len(df.columns), 1), df.columns, rotation=80)
-    #     plt.show()
-    #     return plt
+        plt.figure(figsize=(int(len(df.columns)/4) if len(df.columns)>30 else 10, 10))
+        plt.pcolor(df.isnull(), cmap='Blues_r')
+        plt.yticks([int(el*(len(df)/10)) for el in range(0, 10)])
+        plt.xticks(np.arange(0.5, len(df.columns), 1), df.columns, rotation=80)
+        plt.show()
+        return plt
 
 
 
@@ -217,70 +401,70 @@ class Snaplib:
 
 
    
-    # def cleane(self, df, target=None, verbose=True):
-    #     '''
-    #     drop_duplicates, 
-    #     drop rows with nan in target, 
-    #     drop column with 1 unique value
+    def cleane(self, df, target=None, verbose=True):
+        '''
+        drop_duplicates, 
+        drop rows with nan in target, 
+        drop column with 1 unique value
 
-    #     Use case:
-    #     df = Snaplib().cleane(df, target_name, verbose=True)
-    #     '''
+        Use case:
+        df = Snaplib().cleane(df, target_name, verbose=True)
+        '''
 
-    #     if not isinstance(df, pd.core.frame.DataFrame):
-    #         raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
-    #     if target is not None:
-    #         if type(target) != str:
-    #             raise TypeError('target must be of str type.')
-    #     if type(verbose) != int and type(verbose) != bool:
-    #         raise TypeError('verbose must be of int type or bool.')
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if target is not None:
+            if type(target) != str:
+                raise TypeError('target must be of str type.')
+        if type(verbose) != int and type(verbose) != bool:
+            raise TypeError('verbose must be of int type or bool.')
         
-    #     # DROP DUPLICATES
-    #     start_shape = df.shape
-    #     if verbose:
-    #         print(f'Start shape: {start_shape}\n\n')
-    #         print(f'DROP DUPLICATES:')
-    #     df.drop_duplicates(inplace=True)
-    #     df.reset_index(drop=True, inplace=True)
-    #     dr_dupl_shape = df.shape
-    #     if verbose:
-    #         print(f'{start_shape[0] - dr_dupl_shape[0]} rows have been dropped')
-    #         print(f'shape: {dr_dupl_shape}\n')
+        # DROP DUPLICATES
+        start_shape = df.shape
+        if verbose:
+            print(f'Start shape: {start_shape}\n\n')
+            print(f'DROP DUPLICATES:')
+        df.drop_duplicates(inplace=True)
+        df.reset_index(drop=True, inplace=True)
+        dr_dupl_shape = df.shape
+        if verbose:
+            print(f'{start_shape[0] - dr_dupl_shape[0]} rows have been dropped')
+            print(f'shape: {dr_dupl_shape}\n')
 
-    #     # DROP COLUMNS with 1 unique value
-    #         print('DROP COLUMNS with 1 unique value:')
+        # DROP COLUMNS with 1 unique value
+            print('DROP COLUMNS with 1 unique value:')
             
-    #         count_drop_columns = 0
-    #     for col in df.columns:
-    #         unique_array = np.array(df[col].unique())
+            count_drop_columns = 0
+        for col in df.columns:
+            unique_array = np.array(df[col].unique())
 
-    #         if len(unique_array) == 1:
-    #             count_drop_columns += 1
-    #             df.drop([col], inplace=True, axis=1)
-    #             if verbose:
-    #                 print(f'column "{col}" cnontains 1 unique value - has been dropped')
-    #         elif len(unique_array) == 2 and np.any(pd.isnull(df[col])):
-    #             if verbose:
-    #                 print(f'!!! column "{col}" cnontains 1 unique value and np.nan')
+            if len(unique_array) == 1:
+                count_drop_columns += 1
+                df.drop([col], inplace=True, axis=1)
+                if verbose:
+                    print(f'column "{col}" cnontains 1 unique value - has been dropped')
+            elif len(unique_array) == 2 and np.any(pd.isnull(df[col])):
+                if verbose:
+                    print(f'!!! column "{col}" cnontains 1 unique value and np.nan')
         
-    #     if verbose:
-    #         print(f'{count_drop_columns} columns have been dropped')
-    #         print(f'shape: {df.shape}\n')
+        if verbose:
+            print(f'{count_drop_columns} columns have been dropped')
+            print(f'shape: {df.shape}\n')
 
-    #     # DROP ROWS with NaN IN TARGET
-    #     if target:
-    #         if verbose:
-    #             print('DROP ROWS with NaN IN TARGET:')
-    #         nan = df[df[target].isnull()]
-    #         indeces = list(nan.index)
-    #         if verbose:
-    #             print(f'{len(indeces)} rows have been dropped')
-    #         df = df.drop(df.index[indeces])
-    #         df.reset_index(drop=True, inplace=True)
-    #     if verbose:
-    #         print(f'shape: {df.shape}\n')
-    #         print(f'\nFinish shape: {df.shape}\n')
-    #     return df
+        # DROP ROWS with NaN IN TARGET
+        if target:
+            if verbose:
+                print('DROP ROWS with NaN IN TARGET:')
+            nan = df[df[target].isnull()]
+            indeces = list(nan.index)
+            if verbose:
+                print(f'{len(indeces)} rows have been dropped')
+            df = df.drop(df.index[indeces])
+            df.reset_index(drop=True, inplace=True)
+        if verbose:
+            print(f'shape: {df.shape}\n')
+            print(f'\nFinish shape: {df.shape}\n')
+        return df
 
     
     
@@ -1518,6 +1702,8 @@ class Snaplib:
         device must be "cpu" or "gpu". Sometime small datasets work faster with cpu.
         if set verbose = if True algorithm runs cross validation tests and print results of tests for decision making.
         discrete_columns = ['col_name_1', 'col_name_2', 'col_name_3', 'etc']
+
+        TESTS on https://www.kaggle.com/code/artyomkolas/nan-prediction-in-progress/notebook
         '''
 
         if not isinstance(df_0, pd.core.frame.DataFrame):
